package com.jg.softbus.naservice;

import com.jg.softbus.common.SoftbusErrCode;
import com.jg.softbus.network.callback.ILNNCallback;
import com.jg.softbus.network.callback.INodeStateCb;
import com.jg.softbus.network.node.*;
import com.jg.softbus.network.protocol.ConnectionAddr;

import java.io.Closeable;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * 组网逻辑
 */
public class LNNService implements ILNNCallback, INodeStateCb, Closeable {
    /**
     * 是否出于停止状态，处于停止状态将不再接收外部调用处理
     */
    private AtomicBoolean isStop = new AtomicBoolean(false);
    /**
     * 包名
     */
    private final String pkgName;
    /**
     * 获取包名
     * @return
     */
    public String getPkgName() {
        return pkgName;
    }

    /**
     * 组网状态
     */
    private ConcurrentHashMap<ConnectionAddr, LNNResult> addrLNNResult = new ConcurrentHashMap<>(8);

    public void addLNNResult(ConnectionAddr addr, LNNResult result) {
        addrLNNResult.put(addr, result);
    }

    public LNNResult getLNNResult(ConnectionAddr addr) {
        return addrLNNResult.get(addr);
    }

    public ConnectionAddr tryFindConnectAddr(String networkId) {
        for(Map.Entry<ConnectionAddr, LNNResult> entry: addrLNNResult.entrySet()) {
            ConnectionAddr addr = entry.getKey();
            LNNResult rs = entry.getValue();
            if(rs.getNetworkId().equals(networkId)) {
                return addr;
            }
        }
        return null;
    }

    public boolean isStop() {
        return isStop.get();
    }

    public LNNService(String pkgName) {
        this.pkgName = pkgName;
    }

    static {
        System.loadLibrary("SoftbusLNN");
    }

    /**
     * 发起组网
     * @param targetAddr
     * @throws Exception
     */
    public void joinLNN(ConnectionAddr targetAddr) throws Exception {
        joinLNN(targetAddr, this);
    }

    /**
     * 发起组网
     * @param targetAddr
     * @param callback
     * @throws Exception
     */
    public void joinLNN(ConnectionAddr targetAddr, ILNNCallback callback) throws Exception {
        if(isStop.get()) {
            throw new Exception("LNN service is stop");
        }
        int ret = joinLNN(pkgName, targetAddr, this);
        if(ret != 0){
            throw new Exception(SoftbusErrCode.getByErrCode(ret).getErrMsg());
        }
    }
    /**
     * 发起组网
     * @param targetAddr see {@link ConnectionAddr}
     * @param callback
     * @return
     */
    private native int joinLNN(String pkgName, ConnectionAddr targetAddr, ILNNCallback callback);

    /**
     * 离开网络
     * @param networkId
     * @throws Exception
     */
    public void leaveLNN(String networkId) throws Exception {
        leaveLNN(networkId, this);
    }

    /**
     * 离开网络
     * @param networkId
     * @param callback
     * @throws Exception
     */
    public void leaveLNN(String networkId, ILNNCallback callback) throws Exception {
        if(isStop.get()) {
            throw new Exception("LNN service is stop");
        }
        int ret = leaveLNN(pkgName, networkId, callback);
        if(ret != 0){
            throw new Exception(SoftbusErrCode.getByErrCode(ret).getErrMsg());
        }
    }
    /**
     * 离开网络
     * @param networkId
     * @return
     */
    private native int leaveLNN(String packageName, String networkId, ILNNCallback callback);

    public void registerNodeStateListener() throws Exception {
        registerNodeStateListener(this);
    }
    public void registerNodeStateListener(INodeStateCb callback) throws Exception {
        if(isStop.get()) {
            throw new Exception("LNN service is stop");
        }
        int ret = regNodeDeviceStateCb(pkgName, callback);
        if(ret != 0){
            throw new Exception(SoftbusErrCode.getByErrCode(ret).getErrMsg());
        }
    }
    /**
     * 查询组网状态
     * @param packageName
     * @return
     */
    private native int regNodeDeviceStateCb(String packageName, INodeStateCb callback);

    /**
     * 注销监听事件
     * @throws Exception
     */
    public void unRegisterNodeStateListener() throws Exception {
        unRegisterNodeStateListener(this);
    }

    /**
     * 注销监听事件
     * @param callback
     * @throws Exception
     */
    public void unRegisterNodeStateListener(INodeStateCb callback) throws Exception {
        if(isStop.get()) {
            throw new Exception("LNN service is stop");
        }
        int ret = unRegNodeDeviceStateCb(callback);
        if(ret != 0){
            throw new Exception(SoftbusErrCode.getByErrCode(ret).getErrMsg());
        }
    }
    /**
     * 注销节点状态事件回调
     * @param callback
     * @return
     */
    private native int unRegNodeDeviceStateCb(INodeStateCb callback);

    @Override
    public void onJoinLNNResult(ConnectionAddr addr, String networkId, int retCode) {
        //参与的网络
        if(retCode == 0){
            //组网成功
            System.out.println("组网成功，组网的网络id："+networkId);
        } else {
            System.out.println("组网失败，失败原因："+SoftbusErrCode.getByErrCode(retCode).getErrMsg());
        }
        addLNNResult(addr, new LNNResult(networkId, retCode));
    }

    @Override
    public void onLeaveLNNResult(String networkId, int retCode) {
        if(retCode == 0) {
            System.out.println("退出网络："+networkId+"成功");
        } else {
            System.out.println("退出网络："+networkId+"失败，失败代码："+retCode);
        }
        ConnectionAddr addr = tryFindConnectAddr(networkId);
        if(addr != null){
            addrLNNResult.remove(addr);
        }
    }

    @Override
    public void onNodeOnline(NodeBasicInfo info) {
        System.out.println(String.format("设备%s连接上网络%s", info.getDeviceName(), info.getNetworkId()));
    }

    @Override
    public void onNodeOffline(NodeBasicInfo info) {
        System.out.println(String.format("设备%s离开网络%s", info.getDeviceName(), info.getNetworkId()));
    }

    @Override
    public void onNodeBasicInfoChanged(NodeBasicInfoType type, NodeBasicInfo info) {
        System.out.println(String.format("设备%s在网络%s上发生类型变更，当前类型%s", info.getDeviceName(), info.getNetworkId(), type.name()));
    }

    @Override
    public void onNodeStatusChanged(NodeStatusType type, NodeStatus status) {
        System.out.println(String.format("节点%s发生变化，当前节点状态：%s", type.name(), status.toString()));
    }

    @Override
    public int getEvents() {
        return EVENT_NODE_STATE_ONLINE | EVENT_NODE_STATE_OFFLINE;
    }

    @Override
    public void close() throws IOException {
        if(isStop.compareAndSet(false, true)) {
            if(!addrLNNResult.isEmpty()) {
                for(Map.Entry<ConnectionAddr, LNNResult> entry: addrLNNResult.entrySet()) {
                    ConnectionAddr addr = entry.getKey();
                    LNNResult rs = entry.getValue();
                    try {
                        leaveLNN(rs.getNetworkId());
                    } catch (Exception e) {
                    }
                }
            }
            destroy();
            addrLNNResult.clear();
        }
    }

    /**
     * 销毁对象
     */
    private native void destroy();
}
