/** 
* Copyright (c) 2022-2024. jinggezhihui. All rights reserved.  
* FileName: Discovery.java 
* Author: marunming
* Email: runming56@163.com
* Date: 2023年10月26日 下午2:05:19
* version: V1.0
* LastModified: 2023年10月26日 下午2:05:19   
*/ 


package com.jg.softbus.naservice;

import com.jg.softbus.common.SoftbusErrCode;
import com.jg.softbus.discovery.PublishInfo;
import com.jg.softbus.discovery.callback.IPublishCallback;
import com.jg.softbus.discovery.callback.IPublishCb;
import com.jg.softbus.discovery.enums.PublishResult;

import java.io.Closeable;
import java.io.IOException;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

/**
* 发布服务
*
* @Author marunming
* @ClassName: PublishService
* @Email runming56@163.com
* @Date 2023年10月26日 下午2:05:19 
*/
public class PublishService implements IPublishCallback, IPublishCb, Closeable {
	/**
	 * 包名
	 */
	private final String pkgName;
	

	private AtomicBoolean isStop = new AtomicBoolean(false);
	/**
	 * 默认进入类型 1-publishService; 2-publishLNN
	 */
	private AtomicInteger entryType = new AtomicInteger(0);
	/**
	 * 发布id及其状态维护
	 */
	private ConcurrentHashMap<Integer, PublishResult> publishIdState = new ConcurrentHashMap<>(16);

	public PublishService(String pkgName) {
		this.pkgName = pkgName;
	}

	/**
	 * 包名
	 * @return
	 */
	public String getPkgName() {
		return pkgName;
	}

	/**
	 * 更新publishId对应的发布状态
	 * @param publishId 发布id
	 * @param reason 当前状态
	 */
	public void updateState(Integer publishId, int reason) {
		publishIdState.put(publishId, PublishResult.getForValue(reason));
	}

	/**
	 * 查询publishId的发布状态
	 * @param publishId
	 * @return -1表示未找到可能在处理中未回调告知也可能是已经被注销了；0-成功，其他为异常
	 */
	public int getPublishIdState(Integer publishId) {
		if(publishIdState.containsKey(publishId)) {
			return publishIdState.get(publishId).valueOf();
		}
		return -1;
	}

	static {
		System.loadLibrary("SoftbusPublish");
	}
	/**
	 * 发布服务
	 * @param publishInfo
	 * @return
	 * @throws Exception
	 */
	@Deprecated
	public void publishService(PublishInfo publishInfo) throws Exception {
		publishService(publishInfo, this);
	}

	/**
	 * 发布服务
	 * @param publishInfo
	 * @return
	 * @throws Exception
	 */
	@Deprecated
	public void publishService(PublishInfo publishInfo, IPublishCallback callback) throws Exception {
		if(isStop.get()) {
			throw new Exception("publish service is stop");
		}
		if(entryType.get()==2) {
			throw new Exception("method is not common. please call publishLNN method.");
		}
		entryType.set(1);
		//提交发布请求
		int rs = publishService(pkgName, publishInfo, callback);
		if(rs!= 0){
			//抛出异常
			throw new Exception(Objects.requireNonNull(SoftbusErrCode.getByErrCode(rs)).getErrMsg());
		}
	}

	private void stopPublishByPublishId(int publishId) {
		if(publishId<=0) {
			return;
		}
		if(entryType.get()==1) {
			try {
				stopPublish(publishId);
			} catch (Exception e) {
				//不做任何处理
			}
		} else if(entryType.get()==2) {
			try {
				stopPublishLNN(publishId);
			} catch (Exception e) {
				//不做任何处理
			}
		}
	}

	/**
	 * 发布服务
	 * @param packageName
	 * @param info
	 * @throws Exception
	 *
	 */
	@Deprecated
	private native int publishService(String packageName, PublishInfo info, IPublishCallback callback) throws Exception;

	/**
	 *
	 * @param publishId
	 * @throws Exception
	 *
	 */
	@Deprecated
	public void stopPublish(Integer publishId) throws Exception {
		//停止发布程序
		int res = unPublishService(pkgName, publishId);
		System.out.println("停止发布服务指令完成， 返回值："+res);
		publishIdState.remove(publishId);
		if(res!= 0){
			//抛出异常
			throw new Exception(SoftbusErrCode.getByErrCode(res).getErrMsg());
		}
	}
	/**
	 * 注销服务
	 * @param packageName 包名
	 * @param publishId
	 *
	 */
	@Deprecated
	private native int unPublishService(String packageName, int publishId) throws Exception;
	/**
	 * 发布服务
	 * @param publishInfo
	 * @return
	 */
	public void publishLNN(PublishInfo publishInfo) throws Exception {
		publishLNN(publishInfo, this);
	}

	/**
	 * 发布服务
	 * @param publishInfo
	 * @return
	 */
	public void publishLNN(PublishInfo publishInfo, IPublishCb callback) throws Exception {
		if(isStop.get()) {
			throw new Exception("publish service is stop");
		}
		if(entryType.get()==1) {
			throw new Exception("method is not common. please call publishService method.");
		}
		entryType.set(2);
		int rs = publishLNN(pkgName, publishInfo, callback);
		if(rs!= 0) {
			//抛出异常
			throw new Exception(SoftbusErrCode.getByErrCode(rs).getErrMsg());
		}
	}

	/**
	 * 发布服务
	 * @param packageName
	 * @param info
	 * @return
	 * @throws Exception
	 */
	private native int publishLNN(String packageName, PublishInfo info, IPublishCb callback) throws Exception;

	/**
	 * 停止服务发布
	 * @param publishId
	 * @throws Exception
	 */
	public void stopPublishLNN(Integer publishId) throws Exception {
		//停止发布程序
		int res = stopPublishLNN(pkgName, publishId);
		publishIdState.remove(publishId);
		if(res!= 0){
			//抛出异常
			throw new Exception(SoftbusErrCode.getByErrCode(res).getErrMsg());
		}
	}

	/**
	 * 停止发布服务
	 * @param packageName
	 * @param publishId
	 * @return
	 * @throws Exception
	 *
	 */
	private native int stopPublishLNN(String packageName, int publishId) throws Exception;

	@Override
	public void close() throws IOException {
		if(isStop.compareAndSet(false, true)) {
			System.out.println("开始停止发布服务.....");
			//迭代循环关闭所有发布通道
			for(Map.Entry<Integer, PublishResult> entry : publishIdState.entrySet()) {
				Integer publishId = entry.getKey();
				stopPublishByPublishId(publishId);
			}
			destroy();
			if(!publishIdState.isEmpty()) {
				publishIdState.clear();
			}
			entryType.set(0);
			System.out.println("开始停止发布服务.....");
		}
	}
	private native void destroy();

	@Override
	public void onPublishSuccess(int publishId) {
		if(isStop.get()) {
			return;
		}
		//更新发布状态
		this.updateState(publishId, 0);
		System.out.println("发布成功");
	}

	@Override
	public void onPublishFail(int publishId, int reason) {
		if(isStop.get()) {
			return;
		}
		//发布失败
		this.updateState(publishId, reason);
		System.out.println("服务发布失败，发布id："+publishId+",失败返回信息：");
	}

	@Override
	public void onPublishResult(int publishId, int reason) {
		if(isStop.get()) {
			return;
		}
		this.updateState(publishId, reason);
		System.out.println("服务发布完成，发布id："+publishId+", 返回状态："+reason);
	}
}
