package com.jg.softbus.naservice;

import com.jg.softbus.common.SoftbusErrCode;
import com.jg.softbus.discovery.enums.PublishResult;
import com.jg.softbus.manager.SessionManager;
import com.jg.softbus.model.SessionInfo;
import com.jg.softbus.session.*;
import com.jg.softbus.session.callback.IFileReceiveListener;
import com.jg.softbus.session.callback.IFileSendListener;
import com.jg.softbus.session.callback.ISessionListener;

import java.io.Closeable;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * session传输处理
 */
public class SessionService implements ISessionListener, Closeable {

    private final String pkgName;
    /**
     * 自由sessionName
     */
    private final String sessionName;
    /**
     * 维护连接的session信息
     */
    private final SessionManager connSessions;
    /**
     * groupId
     */
    private final String groupId;


    private AtomicBoolean isStop = new AtomicBoolean(false);

    public boolean isStop() {
        return isStop.get();
    }

    public SessionService(String pkgName, String sessionName) {
        this(pkgName, sessionName, UUID.randomUUID().toString().replaceAll("-",""));
    }

    public SessionService(String pkgName, String sessionName, String groupId) {
        this.pkgName = pkgName;
        this.sessionName = sessionName;
        this.groupId = groupId;
        this.connSessions = new SessionManager();
    }

    static {
        System.loadLibrary("SoftbusSession");
    }

    /**
     * 获取session信息
      * @return
     */
    public SessionManager getSessionManager(){
        return connSessions;
    }
    /**
     * 创建session服务，调用类自己维护session名称
     * 基于包名构建session服务
     */
    public void createSessionServer() throws Exception {
        createSessionServer(this);
    }
    /**
     * 创建session服务，调用类自己维护session名称
     * 基于包名构建session服务
     */
    public void createSessionServer(ISessionListener listener) throws Exception {
        int rs = createSessionServer(pkgName, sessionName, listener);
        if(rs !=0 ) {
            throw new Exception("create session server failed!");
        }
    }

    /**
     * 创建会话服务
     * @param packageName
     * @param sessionName
     * @return 返回sessionId
     */
    private native int createSessionServer(String packageName, String sessionName, ISessionListener listener);

    /**
     * 打开会话服务
     * @param targetSessionId
     * @param targetNetworkId
     * @param attr
     * @throws Exception
     */
    public void openSession(int targetSessionId, String targetNetworkId, SessionAttribute attr) throws Exception {
        if (Objects.isNull(connSessions.getSession(targetSessionId))) {
            throw new Exception("sessionId error.");
        }
        int rs = openSession(sessionName, connSessions.getSession(targetSessionId).getSessionName(), targetNetworkId, groupId, attr);
        if (rs != 0 ) {
            throw new Exception(Objects.requireNonNull(SoftbusErrCode.getByErrCode(rs)).getErrMsg());
        }
    }

    /**
     * 创建会话
     * @param sessionName
     * @param targetSessionName
     * @param targetNetworkId
     * @param groupId
     * @param attr session属性字符串，具体类型参见：{@link SessionAttribute}
     * @return
     */
    private native int openSession(String sessionName, String targetSessionName, String targetNetworkId, String groupId, SessionAttribute attr);

    /**
     * 发送字节数据
     * @param sessionId
     * @param data
     * @param len
     * @return
     */
    public native int sendBytes(int sessionId, byte[] data, int len);

    /**
     * 发送消息数据
     * @param sessionId
     * @param data
     * @param len
     * @return
     */
    public native int sendMessage(int sessionId, String data, int len);

    /**
     * 发送流数据
     * @param sessionId
     * @param data
     * @param ext
     * @param param
     * @return
     */
    public native int sendStream(int sessionId, StreamData data, StreamData ext, StreamFrameInfo param);


    /**
     * 发送文件
     * @param sessionId
     * @param sFileList
     * @param dFileList
     * @param fileCnt
     * @return
     */
    public native int sendFile(int sessionId, String[] sFileList, String[] dFileList, int fileCnt);

    /**
     * 设置文件发送监听器
     * @param fileSendListener
     * @throws Exception
     */
    public void setFileSendListener(IFileSendListener fileSendListener) throws Exception {
        int rs = setFileSendListener(pkgName, sessionName, fileSendListener);
        if(rs !=0 ) {
            throw new Exception(Objects.requireNonNull(SoftbusErrCode.getByErrCode(rs)).getErrMsg());
        }
    }
    /**
     * 注册文件发送回调
     * @param packageName
     * @param sessionName
     * @return
     */
    private native int setFileSendListener(String packageName, String sessionName, IFileSendListener fileSendListener);

    /**
     * 注册接收文件监听器
     * @param fileReceiveListener 文件接收监听器
     * @param directory 文件根目录
     * @throws Exception
     */
    public void setFileReceiveListener(IFileReceiveListener fileReceiveListener, String directory) throws Exception {
        int rs = setFileReceiveListener(pkgName, sessionName, fileReceiveListener, directory);
        if(rs !=0 ) {
            throw new Exception(Objects.requireNonNull(SoftbusErrCode.getByErrCode(rs)).getErrMsg());
        }
    }

    /**
     * 注册文件接收回调
     * @param packageName
     * @param sessionName
     * @param fileReceiveListener 文件接收监听器
     * @return
     */
    private native int setFileReceiveListener(String packageName, String sessionName, IFileReceiveListener fileReceiveListener, String rootDir);

    /**
     * 关闭会话
     * @param sessionId
     */
    public native void closeSession(int sessionId);

    /**
     * 删除会话服务
     * @param packageName
     * @param sessionName
     * @return
     */
    public native int removeSessionServer(String packageName, String sessionName);

    @Override
    public int onSessionOpened(int sessionId, int result) {
        if(result == 0) {
           String sessionName = getPeerSessionName(sessionId);
           String networkId = getPeerDeviceId(sessionId);
           connSessions.addSession(new SessionInfo(sessionId, sessionName, networkId, true));
        }
        return 0;
    }

    /**
     * 获取设备id
     * @param sessionId
     * @return
     */
    private native String getPeerDeviceId(int sessionId);

    public String getSessionName() {
        return sessionName;
    }

    private native String getPeerSessionName(int sessionId);

    @Override
    public void onSessionClosed(int sessionId) {
        connSessions.remove(sessionId);
    }

    @Override
    public void onBytesReceived(int sessionId, byte[] data, int dataLen) {
        System.out.printf("接收到来源于%s的消息，消息内容：%s%n", connSessions.getSession(sessionId).toString(), new String(data, StandardCharsets.UTF_8));
    }

    @Override
    public void onMessageReceived(int sessionId, String data, int dataLen) {
        System.out.printf("接收到来源于%s的消息，消息内容：%s%n", connSessions.getSession(sessionId).toString(), data);
    }

    @Override
    public void onStreamReceived(int sessionId, StreamData data, StreamData ext, StreamFrameInfo param) {
        //TODO 接收数据流处理
    }

    @Override
    public void onQosEvent(int sessionId, int eventId, int tvCount, QosTv[] tvList) {
        System.out.printf("出发qos事件：sessionId: %d, eventId: %d, tvCount: %d, tvList: %s", sessionId, eventId, tvCount, Arrays.toString(tvList));
    }

    @Override
    public void close() throws IOException {
        if(isStop.compareAndSet(false, true)) {
            //销毁session信息
            if(connSessions.getSessions()!=null && !connSessions.getSessions().isEmpty()) {
                System.out.println("关闭session连接");
                connSessions.getSessions().forEach(session -> closeSession(session.getSessionId()));
            }
            System.out.println("移除session服务");
            //移除session服务
            removeSessionServer(pkgName, sessionName);
            destroy();
            connSessions.clear();
        } else {
            System.out.println("状态关闭中....");
        }
    }

    /**
     * 销毁对象
     */
    private native void destroy();
}
