package com.jg.softbus.naservice;

import com.jg.softbus.common.SoftbusErrCode;
import com.jg.softbus.device.DeviceInfo;
import com.jg.softbus.discovery.SubscribeInfo;
import com.jg.softbus.discovery.callback.IDiscoveryCallback;
import com.jg.softbus.discovery.callback.IRefreshCallback;
import com.jg.softbus.discovery.enums.RefreshResult;
import com.jg.softbus.network.node.NodeBasicInfo;

import java.io.Closeable;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 订阅服务
 */
public class SubscribeService implements IDiscoveryCallback, IRefreshCallback, Closeable {
    /**
     * 包名
     */
    private final String pkgName;
    /**
     * 是否出于停止状态，处于停止状态将不再接收外部调用处理
     */
    private AtomicBoolean isStop = new AtomicBoolean(false);
    /**
     * 发现的设备列表
     */
    private CopyOnWriteArrayList<DeviceInfo> devices = new CopyOnWriteArrayList<>();
    /**
     * 订阅状态
     */
    private ConcurrentHashMap<Integer, RefreshResult> refreshIdState = new ConcurrentHashMap<>(8);
    /**
     * 进入类型； 1-discovery 2-refreshLNN
     */
    private AtomicInteger entryType = new AtomicInteger(0);
    /**
     * 获取包名
     * @return
     */
    public String getPkgName() {
        return pkgName;
    }

    /**
     * 更新订阅id的状态
     * @param refreshId
     * @param reason
     */
    public void updateState(Integer refreshId, int reason) {
        refreshIdState.put(refreshId, RefreshResult.getForValue(reason));
    }

    /**
     * 获取订阅id的状态
     * @param refreshId
     * @return -1-未找到可能在进行中可能已经被移除了；0-成功状态；其他为异常
     */
    public int getRefreshIdState(Integer refreshId) {
        if(refreshIdState.containsKey(refreshId)) {
            return  refreshIdState.get(refreshId).valueOf();
        } else {
            return -1;
        }
    }

    public SubscribeService(String pkgName) {
        this.pkgName = pkgName;
    }
    static {
        System.loadLibrary("SoftbusSubscribe");
    }

    /**
     * 发现服务请求
     * @param subscribeInfo
     * @return
     * @throws Exception
     * @deprecated please use {@link #refreshLNN(SubscribeInfo)}
     */
    @Deprecated
    public void discoveryStart(SubscribeInfo subscribeInfo) throws Exception {
        discoveryStart(subscribeInfo, this);
    }

    /**
     * 发现服务请求
     * @param subscribeInfo
     * @param callback
     * @return
     * @throws Exception
     * @deprecated please use {@link #refreshLNN(SubscribeInfo, IRefreshCallback)}
     */
    @Deprecated
    public void discoveryStart(SubscribeInfo subscribeInfo, IDiscoveryCallback callback) throws Exception {
        if(isStop.get()) {
            throw new Exception("subscribe service has stop");
        }
        if(entryType.get()==2) {
            throw new Exception("method is not common. please call refreshLNN method.");
        }
        entryType.set(1);
        int ret = discoveryStart(pkgName, subscribeInfo, callback);
        if (ret != 0) {
            throw new Exception(SoftbusErrCode.getByErrCode(ret).getErrMsg());
        }
    }

    /**
     *  发现服务请求
     * @param packageName
     * @param subscribeInfo
     * @param callback
     * @return
     * @throws Exception
     * @deprecated please use {@link #refreshLNN(String, SubscribeInfo, IRefreshCallback)}
     */
    @Deprecated
    private native int discoveryStart(String packageName, SubscribeInfo subscribeInfo, IDiscoveryCallback callback) throws Exception;
    /**
     * 发现设备
     * @param subscribeInfo
     * @return
     */
    public void refreshLNN(SubscribeInfo subscribeInfo) throws Exception {
        refreshLNN(subscribeInfo, this);
    }

    /**
     * 发现设备
     * @param subscribeInfo
     * @return
     */
    public void refreshLNN(SubscribeInfo subscribeInfo, IRefreshCallback callback) throws Exception {
        if(isStop.get()) {
            throw new Exception("subscribe service has stop");
        }
        if(entryType.get()==1) {
            throw new Exception("method is not common. please call discoveryStart method.");
        }
        entryType.set(2);
        int res = refreshLNN(pkgName, subscribeInfo, callback);
        if (res != 0) {
            throw new Exception(SoftbusErrCode.getByErrCode(res).getErrMsg());
        }
    }

    /**
     * 发现服务请求
     * @param packageName 应用包名
     * @param subscribeInfo json字符串，参见{@link SubscribeInfo}
     * @param callback 回调对象
     * @return
     */
    private native int refreshLNN(String packageName, SubscribeInfo subscribeInfo, IRefreshCallback callback) throws Exception;

    /**
     * 停止订阅
     * @param refreshId 同SubscribeInfo中的subcribeId
     * @throws Exception
     * @deprecated please use {@link #stopRefreshLNN(int)}
     */
    public void stopDiscovery(int refreshId) throws Exception {
        int res = stopDiscovery(pkgName, refreshId);
        System.out.println("停止发现服务指令完成，refreshId:"+refreshId);
        refreshIdState.remove(refreshId);
        if (res != 0) {
            throw new Exception(SoftbusErrCode.getByErrCode(res).getErrMsg());
        }
    }
    /**
     * 停止发现服务
     * @param packageName
     * @param refreshId 同SubscribeInfo中的subcribeId
     * @throws Exception
     * @return
     * @deprecated please use {@link #stopRefreshLNN(String, int)}
     */
    @Deprecated
    private native int stopDiscovery(String packageName, int refreshId) throws Exception;

    /**
     * 停止发现服务
     * @param refreshId 同SubscribeInfo中的subcribeId
     * @throws Exception
     */
    public void stopRefreshLNN(int refreshId) throws Exception {
        int res = stopRefreshLNN(pkgName, refreshId);
        refreshIdState.remove(refreshId);
        if (res != 0) {
            throw new Exception(SoftbusErrCode.getByErrCode(res).getErrMsg());
        }
    }
    /**
     * 停止发现服务
     * @param packageName
     * @param refreshId 同SubscribeInfo中的subcribeId
     * @return
     */
    private native int stopRefreshLNN(String packageName, int refreshId) throws Exception;

    @Override
    public void close() throws IOException {
        if(isStop.compareAndSet(false, true)) {
            System.out.println("开始停止订阅服务");
            //停止状态
            for (Map.Entry<Integer, RefreshResult> entry : refreshIdState.entrySet()) {
                stopSubscribeByRefreshId(entry.getKey());
            }
            destroy();
            if (!refreshIdState.isEmpty()) {
                refreshIdState.clear();
            }
            devices.clear();
            entryType.set(0);
            System.out.println("订阅服务停止完成");
        }
    }

    /**
     * 销毁指定连接
     * @param refreshId
     */
    private void stopSubscribeByRefreshId(int refreshId) {
        if(refreshId<=0) {
            return;
        }
        if(entryType.get()==1) {
            try {
                stopDiscovery(refreshId);
            } catch (Exception e) {
                //不做任何处理
            }
        } else if (entryType.get()==2) {
            try {
                stopRefreshLNN(refreshId);
            } catch (Exception e) {
                //不做任何处理
            }
        }
    }

    /**
     * 发现在线网络节点信息
     * @return
     */
    public List<NodeBasicInfo> getOnlineDeviceNodeInfo() throws Exception {
        if(isStop.get()) {
            throw new Exception("subscribe service has stop");
        }
        NodeBasicInfo[] deviceNodes = findDeviceNodes(pkgName);
        try {
            if (deviceNodes != null && deviceNodes.length > 0) {
                return Arrays.stream(deviceNodes).collect(Collectors.toList());
            }
            return Collections.emptyList();
        } finally {
            releaseDeviceNodes(deviceNodes);
        }
    }

    /**
     * 查找在线网络节点
     * @param pkgName
     * @return
     */
    private native NodeBasicInfo[] findDeviceNodes(String pkgName);

    /**
     *  释放设备节点数据
     * @param nodes
     */
    private native void releaseDeviceNodes(NodeBasicInfo[] nodes);

    /**
     * 销毁对象
     */
    private native void destroy();

    @Override
    public void onDeviceFound(DeviceInfo info) {
        if(isStop.get()) {
            //服务停止拒绝接受其他更新处理
            return;
        }
        //自定义存储设备信息
        if(!devices.contains(info)){
            devices.add(info);
        } else {
            //使用最新的状态信息
            devices.remove(info);
            devices.add(info);
        }
    }

    @Override
    public void onDiscoverResult(int refreshId, int reason) {
        if(isStop.get()) {
            //服务停止拒绝接受其他更新处理
            return;
        }
        updateState(refreshId, reason);
        System.out.println("服务订阅结果通知，订阅id："+refreshId+",返回状态码："+reason);
    }

    @Override
    public void onDiscoverFailed(int subscribeId, int reason) {
        if(isStop.get()) {
            //服务停止拒绝接受其他更新处理
            return;
        }
        updateState(subscribeId, reason);
        //更新发布状态
        System.out.println("服务订阅失败，订阅id："+subscribeId+",失败返回状态码："+reason);
    }

    @Override
    public void onDiscoverySuccess(int subscribeId) {
        if(isStop.get()) {
            //服务停止拒绝接受其他更新处理
            return;
        }
        //更新发布状态
        updateState(subscribeId, 0);
        System.out.println("订阅成功");
    }
}
